package it.openutils.magnoliastripes;

import info.magnolia.cms.beans.config.Paragraph;
import info.magnolia.cms.beans.config.ParagraphManager;
import net.sourceforge.stripes.action.ActionBean;
import net.sourceforge.stripes.controller.NameBasedActionResolver;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * ActionResolver that extends <code>NameBasedActionResolver</code>, registering any found Stripe action as a
 * Magnolia paragraph.
 * @author fgiust
 * @version $Id: $
 */
public class MgnlActionResolver extends NameBasedActionResolver
{

    /**
     * Logger.
     */
    private Logger log = LoggerFactory.getLogger(MgnlActionResolver.class);

    /**
     * {@inheritDoc}
     */
    @Override
    protected void addActionBean(Class< ? extends ActionBean> clazz)
    {
        String binding = getUrlBinding(clazz);

        // Only process the class if it's properly annotated
        if (binding != null)
        {
            String dialogName = actionNameToParagraphName(binding);
            registerSpringParagraph(dialogName, binding);
            super.addActionBean(clazz);
        }
    }

    /**
     * Generate a paragraph name from a Stripes binding. This method will take the last token after "/", strip any
     * extension and convert everything to lowercase.
     * @param binding Stripe action binding
     * @return paragraph name
     */
    protected String actionNameToParagraphName(String binding)
    {
        String dialogName = StringUtils.lowerCase(StringUtils.substringBeforeLast(StringUtils.substringAfterLast(
            binding,
            "/"), "."));
        return dialogName;
    }

    /**
     * Registers a Magnolia paragraph which will delegate to a Stripe action.
     * @param name dialog name
     * @param binding Stripes action binding
     */
    @SuppressWarnings("unchecked")
    private void registerSpringParagraph(String name, String binding)
    {
        Paragraph paragraph = new Paragraph();

        paragraph.setName(name);
        paragraph.setTitle("paragraph." + name + ".title");
        paragraph.setDescription("paragraph." + name + ".description");
        paragraph.setDialog(name);
        paragraph.setTemplatePath(binding);
        paragraph.setType("stripes");

        log.info("Registering stripes paragraph {}", paragraph.getName()); //$NON-NLS-1$
        ParagraphManager.getInstance().getParagraphs().put(paragraph.getName(), paragraph);
    }

}
