package it.openutils.magnoliastripes;

import info.magnolia.context.MgnlContext;
import info.magnolia.context.WebContext;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Map;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

import net.sourceforge.stripes.controller.StripesRequestWrapper;
import net.sourceforge.stripes.exception.StripesServletException;


/**
 * Magnolia request wrapper for Stripes actions.
 * @author fgiust
 * @version $Id: $
 */
class StripesParagraphRequestWrapper extends StripesRequestWrapper
{

    /**
     * Hacked servlet path.
     */
    private String servletPath;

    /**
     * Parameter map.
     */
    private Map<String, String[]> parameterMap;

    /**
     * Instantiate a new request wrapper.
     * @param request original HttpServletRequest
     * @param servletPath modified servlet path (matches Stripes binding)
     * @param paragraphsData map containing paragraph attributes
     * @throws StripesServletException if any other error occurs constructing the wrapper
     */
    @SuppressWarnings("unchecked")
    public StripesParagraphRequestWrapper(
        HttpServletRequest request,
        String servletPath,
        Map<String, String[]> paragraphsData) throws StripesServletException
    {
        super(request);
        this.servletPath = servletPath;

        parameterMap = new Hashtable<String, String[]>();
        parameterMap.putAll(request.getParameterMap());
        parameterMap.putAll(paragraphsData);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getServletPath()
    {
        return servletPath;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void setLocale(Locale locale)
    {
        super.setLocale(locale);
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    @Override
    public Enumeration<String> getParameterNames()
    {
        return ((Hashtable) parameterMap).keys();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String[] getParameterValues(String name)
    {
        return parameterMap.get(name);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Map<String, String[]> getParameterMap()
    {

        return parameterMap;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getParameter(String name)
    {
        String[] values = getParameterValues(name);
        if (values != null && values.length > 0)
        {
            return values[0];
        }
        else
        {
            return null;
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public RequestDispatcher getRequestDispatcher(String path)
    {
        return new MagnoliaRequestDispatcher(path);
    }

    /**
     * A {@link RequestDispatcher} that uses {@link WebContext} for including a resource.
     * @author fgiust
     * @version $Id: $
     */
    private static class MagnoliaRequestDispatcher implements RequestDispatcher
    {

        /**
         * The url this RequestDispatcher is bound to.
         */
        private String url;

        /**
         * Created a new MagnoliaRequestDispatcher for the given url.
         * @param url url passed to {@link HttpServletRequest#getRequestDispatcher()}
         */
        public MagnoliaRequestDispatcher(String url)
        {
            this.url = url;
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public void forward(ServletRequest request, ServletResponse response) throws ServletException, IOException
        {
            ((WebContext) MgnlContext.getInstance()).include(url, response.getWriter());
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public void include(ServletRequest request, ServletResponse response) throws ServletException, IOException
        {
            ((WebContext) MgnlContext.getInstance()).include(url, response.getWriter());
        }
    }
}